/*
 * $Id: cgiheader.C,v 1.2 1996/08/25 19:22:54 asf Exp $
 */

/*
   
    This file is part of an HTML generator interface
    Copyright (C) 1996  Allen S. Firstenberg
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "cgiheader.H"

/***************************************************************************/


typedef struct{
  int   num;
  char *string;
} StatusCodes;

StatusCodes code[] = {
  {200, "OK"},
  {201, "CREATED"},
  {202, "Accepted"},
  {203, "Partial Information"},
  {204, "No Response"},
  {301, "Moved"},
  {302, "Found"},
  {303, "Method"},
  {304, "Not Modified"},
  {400, "Bad request"},
  {401, "Unauthorized"},
  {402, "Payment Required"},
  {403, "Forbidden"},
  {404, "Not found"},
  {500, "Internal Error"},
  {501, "Not implemented"},
  {502, "Service temporarily overloaded"},
  {503, "Gateway timeout"},
  {0, NULL}
};

/***************************************************************************/

CgiHeader::CgiHeader(){
  firstline = NULL;
  otherlines = new HtmlText();
}

//---------------------------------------------------------------------------

CgiHeader::~CgiHeader(){
  if( firstline ){
    free( firstline );
  }
}

//---------------------------------------------------------------------------

char *CgiHeader::val(){
  HtmlText buffer;

  if( firstline ){
    buffer.raw( firstline->val() );
  }
  buffer.raw( otherlines->val() );
  buffer.raw( "\r\n" );

  return strdup( buffer.val() );
}

/***************************************************************************/

void CgiHeader::setContentType( char *type ){
  HtmlText *saveto;
  
  if( !firstline ){
    firstline = new HtmlText();
    saveto = firstline;
  } else {
    saveto = otherlines;
  }
  saveto->raw( "Content-Type: " );
  saveto->raw( type );
  saveto->raw( "\r\n" );
}

//---------------------------------------------------------------------------

void CgiHeader::setLocation( char *redirect ){
  HtmlText *saveto;
  
  if( redirect ){
    if( !firstline ){
      firstline = new HtmlText();
      saveto = firstline;
    } else {
      saveto = otherlines;
    }
    saveto->raw( "Location: " );
    saveto->raw( redirect );
    saveto->raw( "\r\n" );
  }
}

//---------------------------------------------------------------------------

void CgiHeader::setStatus( int status, char *reason ){

  firstline = new HtmlText();
  
  char *realreason = NULL;
  if( reason == NULL ){
    // Look it up
    int co = 0;
    while( !realreason && code[co].num ){
      if( code[co].num == status ){
        realreason = code[co].string;
      }
      co++;
    }
    if( !realreason ){
      realreason = "UNKNOWN";
    }
  } else {
    realreason = reason;
  }

  char *buf = (char *)malloc( strlen(realreason) + 20 );
  sprintf( buf, "Status: %d %s\r\n", status, realreason );
  firstline->raw( buf );
}

/***************************************************************************/

/*
 * $Log: cgiheader.C,v $
 * Revision 1.2  1996/08/25 19:22:54  asf
 * Changed method of setting the first line
 *
 * Revision 1.1  1996/08/25 18:42:24  asf
 * Initial revision
 *
 */
