/*
 * $Id: htmlgen.C,v 1.16 1996/08/25 22:51:55 asf Exp $
 */
/*
   
    This file is part of an HTML generator interface
    Copyright (C) 1994  Allen S. Firstenberg
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <sys/stat.h>
#include <unistd.h>

#include "htmlgen.H"

char *align_strings[] = {
"none", "left", "right",
"top", "texttop",
"middle", "absmiddle", "baseline",
"bottom", "absbottom" };

//---------------------------------------------------------------------------
// Constructors/Destructors

HtmlGen::HtmlGen(){
}

//---------------------------------------------------------------------------

HtmlGen::~HtmlGen(){
}

//---------------------------------------------------------------------------
// Titles and headers

void HtmlGen::title( char *text ){
  out( "\n<title>" );
  out( text );
  out( "</title>\n" );
}

//---------------------------------------------------------------------------

void HtmlGen::header( int level, char *text ){
  char s[10];
  
  sprintf( s, "\n<h%d>", level );
  out( s );
  out( text );
  sprintf( s, "</h%d>\n", level );
  out( s );
}

//---------------------------------------------------------------------------
// Already formatted text

void HtmlGen::raw( char *text ){
  /*
   * Just put the text in verbatim
   */

  out( text );
}

int HtmlGen::rawfile( char *filepath ){
  /*
   * Read in lines from a file.  If the file is executable, then run
   * the program and capture its output.
   */
  int ok = 1;
  char buffer[80];
  FILE *infile;

  // @@@ Should check out popen() too!
  if( ok && (infile = fopen( filepath, "r" )) ){
    // Opened file ok
    while( fgets( buffer, 80, infile ) ){
      // file read in
      out( buffer );
    }
    fclose( infile );
  } else {
    // Didn't open
    ok = 0;
  }

  return ok;
}

//---------------------------------------------------------------------------
// Standard formatted text

void HtmlGen::puts( char *text ){
  // \n converted to <p>
  // \r converted to <br>
  int co;
  char s[2];

  s[1] = '\0';   // A 1 character string
  
  for( co=0; co<strlen(text); co++ ){
    if( text[co] == '\n' ){
      out( "<p>\n" );
    } else if( text[co] == '\r' ){
      out( "<br>\n" );
    } else {
      s[0] = text[co];
      out( s );
    }
  }
  
}

//---------------------------------------------------------------------------

void HtmlGen::pputs( char *text ){
  // preformatted text

  out( "\n<PRE>" );
  out( text );
  out( "</PRE>\n" );
  
}

//---------------------------------------------------------------------------

void HtmlGen::para(){
  out( "<p>\n" );
}

//---------------------------------------------------------------------------

void HtmlGen::line(){
  out( "\n<hr>\n" );
}

//---------------------------------------------------------------------------
// Stuff involved with URLs

void HtmlGen::link( char *href, char *text ){
  out( "<a href=\"" );
  out( href );
  out( "\">" );
  out( text );
  out( "</a>" );
}

//---------------------------------------------------------------------------

void HtmlGen::anchor( char *name, char *text ){
  out( "<a name=\"" );
  out( name );
  out( "\">" );
  out( text );
  out( "</a>" );
}

//---------------------------------------------------------------------------

void HtmlGen::image( char *src, char *alt, AlignType align ){
  out( "<img src=\"" );
  out( src );
  out( "\"" );
  if( alt ){
    out( " alt=\"" );
    out( alt );
    out( "\"" );
  }
  if( align != ALIGN_NONE ){
    out( " align=\"" );
    out( align_strings[ align ] );
    out( "\"" );
  }
  out( ">" );
}

//--------------------------------------------------------------------------
// Lists

void HtmlGen::numlist( char *items[] ){
  out( "\n<ol>\n" );
  list_items( items );
  out( "</ol>\n" );
}

//---------------------------------------------------------------------------

void HtmlGen::unnumlist( char *items[] ){
  out( "\n<ul>\n" );
  list_items( items );
  out( "</ul>\n" );
}

/***************************************************************************/

#ifdef UseHtmlList
void HtmlGen::numlist( HtmlList *items ){
  out( "\n<ol>\n" );
  list_items( items );
  out( "</ol>\n" );
}

//---------------------------------------------------------------------------

void HtmlGen::unnumlist( HtmlList *items ){
  out( "\n<ul>\n" );
  list_items( items );
  out( "</ul>\n" );
}

//---------------------------------------------------------------------------

void HtmlGen::desclist( HtmlDList *items ){
  HtmlDNode *n;

  out( "\n<dl>\n" );

  n = (HtmlDNode *)(items->first());
  while( n ){
    out( "<dt> " );
    out( n->value1() );
    out( "\n" );
    out( "<dd> " );
    out( n->value2() );
    out( "\n" );

    n = (HtmlDNode *)(items->next());
  }
  out( "</dl>\n" );
}

#endif // UseHtmlList

/***************************************************************************/
// The comment

void HtmlGen::comment( char *text ){
  out( "<!-- " );
  if( text ){
    out( text );
  }
  out( "-->\n" );
}

//---------------------------------------------------------------------------
// Formatting
void HtmlGen::address( char *text ){
  out( "<address>" );
  out( text );
  out( "</address>" );
}

//---------------------------------------------------------------------------
// Forms
void HtmlGen::form( char *action, Method method, char *contents ){
  // Ignoring ENCTYPE

  out( "\n<form METHOD=" );
  if( method==METHOD_GET ){
    out( "GET" );
  } else {
    out( "POST" );
  }
  out( " ACTION=\"" );
  out( action );
  out( "\">\n" );
  out( contents );
  out( "\n</form>\n" );
}

//---------------------------------------------------------------------------

void HtmlGen::text( char *name, int maxlen, char *default_value ){

  char maxlenstr[40];
  
  out( "<input type=\"text\" name=\"" );
  out( name );
  out( "\"" );
  if( default_value ){
    out( " value=\"" );
    out( default_value );
    out( "\"" );
  }
  if( maxlen ){
    sprintf( maxlenstr, " maxlen=%d", maxlen );
    out( maxlenstr );
  }
  out( ">" );
}

//---------------------------------------------------------------------------

void HtmlGen::text( char *name, int width, int height, char *default_value ){

  char maxsizestr[40];
  
  out( "<input type=\"text\" name=\"" );
  out( name );
  out( "\"" );
  if( default_value ){
    out( " value=\"" );
    out( default_value );
    out( "\"" );
  }
  if( width ){
    if( height ){
      sprintf( maxsizestr, " size=%d,%d", width, height );
    } else {
      sprintf( maxsizestr, " size=%d", width );
    }
    out( maxsizestr );
  }
  out( ">" );
}

//---------------------------------------------------------------------------

void HtmlGen::textarea( char *name, int width, int height,
                        char *default_value ){

  char maxsizestr[40];
  
  out( "<textarea name=\"" );
  out( name );
  out( "\"" );
  sprintf( maxsizestr, " ROWS=%d COLS=%d", height, width );
  out( maxsizestr );
  out( ">" );
  if( default_value ){
    out( default_value );
  }
  out( "</textarea>\n" );
}

//---------------------------------------------------------------------------

void HtmlGen::password( char *name, int maxlen, char *default_value ){

  char maxlenstr[40];
  
  out( "<input type=\"password\" name=\"" );
  out( name );
  out( "\"" );
  if( default_value ){
    out( " value=\"" );
    out( default_value );
    out( "\"" );
  }
  if( maxlen ){
    sprintf( maxlenstr, " maxlen=%d", maxlen );
    out( maxlenstr );
  }
  out( ">" );

}

//---------------------------------------------------------------------------

void HtmlGen::password( char *name, int width, int height,
                        char *default_value ){

  char maxsizestr[40];
  
  out( "<input type=\"password\" name=\"" );
  out( name );
  out( "\"" );
  if( default_value ){
    out( " value=\"" );
    out( default_value );
    out( "\"" );
  }
  if( width ){
    if( height ){
      sprintf( maxsizestr, " size=%d,%d" );
    } else {
      sprintf( maxsizestr, " size=%d" );
    }
    out( maxsizestr );
  }
  out( ">" );
}

//---------------------------------------------------------------------------

void HtmlGen::checkbox( char *name, int checked, char *value ){
  out( "<input type=\"checkbox\" name=\"" );
  out( name );
  out( "\"" );
  if( checked ){
    out( " checked" );
  }
  if( value ){
    out( " value=\"" );
    out( value );
    out( "\"" );
  }
  out( ">" );
}

//---------------------------------------------------------------------------

void HtmlGen::radio( char *name, int checked, char *value ){
  out( "<input type=\"radio\" name=\"" );
  out( name );
  out( "\"" );
  if( checked ){
    out( " checked" );
  }
  if( value ){
    out( " value=\"" );
    out( value );
    out( "\"" );
  }
  out( ">" );
}

//---------------------------------------------------------------------------

void HtmlGen::hidden( char *name, char *value ){
  out( "<input type=hidden" );
  if( value ){
    out( " name=\"" );
    out( name );
    out( "\"" );
  }
  if( value ){
    out( " value=\"" );
    out( value );
    out( "\"" );
  }
  out( ">" );
}

//---------------------------------------------------------------------------

#ifdef UseHtmlList
void HtmlGen::select( char *name, HtmlSList *options, int size, int multiple ){

  char sizestring[40];
  HtmlSNode *n;

  out( "<select name=\"" );
  out( name );
  out( "\"" );
  if( size ){
    sprintf( sizestring, " size=%d", size );
    out( sizestring );
  }
  if( multiple ){
    out( " multiple" );
  }
  out( ">\n" );

  n = (HtmlSNode *)(options->first());
  while( n ){
    out( "<option" );
    if( n->selected() ){
      out( " selected" );
    }
    out( "> " );
    out( n->value() );
    out( "\n" );
    n = (HtmlSNode *)(options->next());
  }

  out( "</select>\n" );
}
#endif

//---------------------------------------------------------------------------
  
void HtmlGen::submit( char *text ){
  out( "<input type=\"submit\"" );
  if( text ){
    out( " value=\"" );
    out( text );
    out( "\"" );
  }
  out( ">" );
}

//---------------------------------------------------------------------------

void HtmlGen::reset( char *text ){
  out( "<input type=\"reset\"" );
  if( text ){
    out( " value=\"" );
    out( text );
    out( "\"" );
  }
  out( ">" );
}

//---------------------------------------------------------------------------

void HtmlGen::isindex(){
  out( "<isindex>\n" );
}

//---------------------------------------------------------------------------
    
void HtmlGen::out( char *text ){
  // Virtual.  What do to with text thats created.  Does not free memory
}

//---------------------------------------------------------------------------
// Convert special characters

int HtmlGen::needs_conversion( char *text ){
  // @@@ Under construction
}

//---------------------------------------------------------------------------

char *HtmlGen::convert( char *text ){
  // Returns newly allocated string
  // @@@ Under construction

}

//---------------------------------------------------------------------------

void HtmlGen::list_items( char *items[] ){
  int co = 0;

  while( items[co] ){
    out( "<li> " );
    out( items[co] );
    out( "\n" );
    co++;
  }
}

//---------------------------------------------------------------------------

#ifdef UseHtmlList
void HtmlGen::list_items( HtmlList *items ){
  HtmlNode *n;

  n = (HtmlNode *)(items->first());
  while( n ){
    out( "<li> " );
    out( n->value() );
    out( "\n" );

    n = (HtmlNode *)(items->next());
  }
}
#endif

//---------------------------------------------------------------------------

#ifdef UseHtmlList
void HtmlGen::applet( char *code, int width, int height,
                      HtmlDList *params, char *nonjavatext,
                      AlignType align,
                      char *codebase, char *name, char *alt,
                      int vspace, int hspace ){

  // Print the header
  char tmp[100];

  // Print the start tag
  out( "<applet code=\"" );
  out( code );
  sprintf( tmp, " width=%d", width );
  out( tmp );
  sprintf( tmp, " height=%d", height );
  out( tmp );
  if( alt ){
    out( " alt=\"" );
    out( alt );
    out( "\"" );
  }
  if( align != ALIGN_NONE ){
    out( " align=\"" );
    out( align_strings[ align ] );
    out( "\"" );
  }
  if( codebase ){
    out( " codebase=\"" );
    out( codebase );
    out( "\"" );
  }
  if( name ){
    out( " name=\"" );
    out( name );
    out( "\"" );
  }
  out( ">\n" );
  
  // Print any parameters
  if( params ){
    HtmlDNode *n;

    n = (HtmlDNode *)(params->first() );
    while( n ){
      out( "<param name=\"" );
      out( n->value1() );
      out( "\" value=\"" );
      out( n->value2() );
      out( "\">\n" );

      n = (HtmlDNode *)(params->next());
    }
  }

  // Print any non-java text
  if( nonjavatext ){
    raw( nonjavatext );
  }

  // Print the close tag
  out( "</applet>\n" );
}
#endif
//---------------------------------------------------------------------------

/*
 * $Log: htmlgen.C,v $
 * Revision 1.16  1996/08/25 22:51:55  asf
 * Fixed problem with rawfile()
 *
 * Revision 1.15  1996/06/23 11:02:26  asf
 * Added applet() function.
 * Added align parameter to the image() function.
 *
 * Revision 1.14  1996/03/25  20:15:46  asf
 * Added rawfile() method
 *
 * Revision 1.13  1995/08/19  09:41:21  asf
 * inserted an ifdef for UseHtmlList around one of the list_items method
 *
 * Revision 1.12  1995/03/15  19:52:28  asf
 * Prefixed some commands with newlines to try and make the pages easier to read
 * (or something) and for some browsers to parse.
 *
 * Revision 1.11  1995/03/06  20:58:53  asf
 * Added ALT tags for immage()
 *
 * Revision 1.10  1995/02/15  20:53:43  asf
 * Fixed spacing problem in textarea.
 *
 * Revision 1.9  1995/01/28  17:22:54  asf
 * Fix to one of the text() methods to correct how the width
 * and height parameters are handled.
 *
 * Revision 1.8  1994/09/06  20:58:42  asf
 * Added isindex()
 *
 * Revision 1.7  1994/08/08  19:50:18  asf
 * Added raw()
 *
 * Revision 1.6  1994/08/06  09:13:04  asf
 * Capitalized the method text for "get" and "post".
 *
 * Revision 1.5  1994/08/03  20:04:57  asf
 * Fixed name/value problems with hidden().
 *
 * Revision 1.4  1994/08/03  16:20:19  asf
 * Added a newline to be sent after a <p> or <br> is generated from a puts().
 *
 * Revision 1.3  1994/07/28  17:50:43  asf
 * Added hidden type and comment type
 *
 * Revision 1.2  1994/07/26  21:27:16  asf
 * Improved parameters for text() to make calls better structured.
 *
 * Revision 1.1  1994/07/26  19:57:49  asf
 * Initial revision
 *
 */
