/*
 * $Id: htmlgen.H,v 1.9 1996/06/23 11:02:50 asf Exp $
 */

/*
   
    This file is part of an HTML generator interface
    Copyright (C) 1994-1996  Allen S. Firstenberg
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef _HTMLGEN_H
#define _HTMLGEN_H

#include <stdio.h>

#ifdef UseHtmlList
#include "htmllist.H"
#endif // UseHtmlList

/***************************************************************************/

enum Method { METHOD_GET, METHOD_POST };
enum AlignType { ALIGN_NONE, ALIGN_LEFT, ALIGN_RIGHT,
                 ALIGN_TOP, ALIGN_TEXTTOP,
                 ALIGN_MIDDLE, ALIGN_ABSMIDDLE, ALIGN_BASELINE,
                 ALIGN_BOTTOM, ALIGN_ABSBOTTOM };

/***************************************************************************/

class HtmlGen{
  
public:
  
  // Constructors/Destructors
  HtmlGen();
  virtual ~HtmlGen();

  // Titles and headers
  void title( char *text );
  void header( int level, char *text );

  // Already formatted text
  void raw( char *text );    // Just put the text in verbatim
  int  rawfile( char *filepath );   // Read in lines from a file
  
  // Standard formatted text
  void puts( char *text );   // \n converted to <p>
  void pputs( char *text );  // preformatted text
  void para();
  void line();
  
  // Stuff involved with URLs
  void link( char *href, char *text );
  void anchor( char *name, char *text );
  void image( char *src, char *alt = NULL, AlignType align = ALIGN_NONE );

  // Lists
  void numlist( char *items[] );
  void unnumlist( char *items[] );
  #ifdef UseHtmlList
  void numlist( HtmlList *items );
  void unnumlist( HtmlList *items );
  void desclist( HtmlDList *items );
  #endif // UseHtmlList

  // Formatting
  void address( char *text );

  // The comment
  void comment( char *text );

  // Forms
  void form( char *action, Method method, char *contents ); // Ignoring ENCTYPE

  void text( char *name, int maxlen=0, char *default_value=NULL );
  void text( char *name, int width, int height, char *default_value=NULL );
  void textarea( char *name, int width=0, int height=0,
                 char *default_value=NULL );
  void password( char *name, int maxlen=0, char *default_value=NULL );
  void password( char *name, int width, int height, char *default_value=NULL );
  void checkbox( char *name, int checked = 0, char *value = NULL );
  void radio( char *name, int checked = 0, char *value = NULL );
  void hidden( char *name, char *value = NULL );
  
  #ifdef UseHtmlList
  void select( char *name, HtmlSList *options, int size=1, int multiple=0 );
  #endif
  
  void submit( char *text = NULL );
  void reset( char *text = NULL );

  void isindex();

  // Java Applets
  #ifdef UseHtmlList
  void applet( char *code, int width, int height,
               HtmlDList *params = NULL, char *nonjavatext = NULL,
               AlignType align = ALIGN_NONE,
               char *codebase = NULL, char *name = NULL, char *alt = NULL,
               int vspace = 1, int hspace = 1 );
  #endif

private:
  // Virtual.  What do to with text thats created.  Does not free memory
  virtual void out( char *text );
  
  // Convert special characters
  int needs_conversion( char *text );
  char *convert( char *text );         // Returns newly allocated string

  // List the array of strings
  void list_items( char *items[] );
  #ifdef UseHtmlList
  void list_items( HtmlList *items );
  #endif
  
};

/***************************************************************************/

#endif

/*
 * $Log: htmlgen.H,v $
 * Revision 1.9  1996/06/23 11:02:50  asf
 * Added applet() function.
 * Added align parameter to the image() function.
 *
 * Revision 1.8  1996/03/25 20:15:29  asf
 * Added rawfile() method
 *
 * Revision 1.7  1995/08/19  09:41:28  asf
 * inserted an ifdef for UseHtmlList around one of the list_items method
 *
 * Revision 1.6  1995/03/06  20:59:06  asf
 * Added ALT tags for image()
 *
 * Revision 1.5  1994/09/06  20:58:53  asf
 * Added isindex()
 *
 * Revision 1.4  1994/08/08  19:50:30  asf
 * Added raw()
 *
 * Revision 1.3  1994/07/28  17:51:01  asf
 * Added hidden type and comment type.
 *
 * Revision 1.2  1994/07/26  21:27:23  asf
 * Improved parameters for text() to make calls better structured.
 *
 * Revision 1.1  1994/07/26  19:59:01  asf
 * Initial revision
 *
 */
