/*
 * list.C
 *
 * Base classes for linked list
 */

/*
 * $Id: list.C,v 1.2 1994/08/24 14:35:11 asf Exp $
 */

/*
   
    This file is a generic list
    Copyright (C) 1993-4  Allen S. Firstenberg
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include"list.H"

Node::Node( Node *prev, Node *next ){
  _prev = prev;
  _next = next;
}


Node *Node::next(){
  return _next;
}


void Node::set_next( Node *next ){
  _next = next;
}


Node *Node::prev(){
  return _prev;
}


void Node::set_prev( Node *prev ){
  _prev = prev;
}

/***************************************************************************/

List::List(){
  _count   = 0;
  _first   = NULL;
  _current = NULL;
  _last    = NULL;
}


Node *List::current(){
  return _current;
}


Node *List::get_first(){
  return _first;
}


Node *List::get_last(){
  return _last;
}


Node *List::first(){
  _current = _first;
  return _first;
}


Node *List::last(){
  _current = _last;
  return _last;
}


Node *List::next(){
  if( _current ){
    _current = _current->next();
  }
  return _current;
}


Node *List::prev(){
  if( _current ){
    _current = _current->prev();
  }
  return _current;
}


unsigned List::count(){
  return _count;
}


void List::append( Node &n ){
  if( _count == 0 ){
    _first = _current = _last = &n;
    _count++;
  } else {
    n.set_prev( _last );
    _last->set_next( &n );
    _last = &n;
    _count++;
  }
}


void List::insert( Node &n ){
  if( _count == 0 ){
    append( n );
  } else {
    n.set_next( _current->next() );
    n.set_prev( _current );
    if( _current == _last ){
      _last = &n;
    } else {
      _current->next()->set_prev( &n );
    }
    _current->set_next( &n );
    _count++;
  }
}


void List::insert_before( Node &n ){
  if( _count == 0 ){
    append( n );
  } else {
    n.set_next( _current );
    n.set_prev( _current->prev() );
    if( _current == _first ){
      _first = &n;
    } else {
      _current->prev()->set_next( &n );
    }
    _current->set_prev( &n );
    _count++;
  }
}


void List::insert_after( Node &n ){
  insert( n );
}


Node *List::remove(){
  Node *die;     /* The node being removed */
  Node *p, *n;   /* The previous and next nodes */

  if( _current ){
    /* If there is a node to remove */
    die = _current;
    p   = _current->prev();
    n   = _current->next();

    if( (!p) && (!n) ){
      /* We now have an empty list
       * Reset and do nothign else
       */
      _count = 0;
      _current = NULL;
      _first   = NULL;
      _last    = NULL;
      return die;
    }

    if( p ){
      /* There was a previous node */
      p->set_next( n );
    } else {
      /* No previous node.  _current was also _first, so update _first */
      _first = n;
    }

    if( n ){
      /* There was a next node */
      n->set_prev( p );
    } else {
      /* No next node.  _current was also _last, so update _last */
      _last = p;
    }

    // Move current to the next node in the list or, if there is no next
    // node, move it to the last node.
    if( n ){
      _current = n;
    } else {
      _current = _last;
    }
    _count--;
    return die;
  }
}

    
/***************************************************************************/

INode::INode( int val, INode *prev, INode *next ):
  Node( prev, next ){
  _value = val;
}


int INode::value(){
  return _value;
}

/*
 * $Log: list.C,v $
 * Revision 1.2  1994/08/24  14:35:11  asf
 * Added insert_previous() and insert_next().
 * Fixed some bugs with inserting at the beginning and end of a list.
 *
 * Revision 1.1  1994/07/26  20:05:37  asf
 * Initial revision
 *
 */
