/*
 * list.H
 *
 * Base classes for linked list
 */

/*
 * $Id: list.H,v 1.2 1994/08/24 14:35:27 asf Exp $
 */

/*
   
    This file is a generic list
    Copyright (C) 1994  Allen S. Firstenberg
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef _LIST_H
#define _LIST_H

#include "standard.H"

class Node{
  protected:
    Node *_next;
    Node *_prev;
    
  public:
    // Constructor
    Node( Node *prev = NULL, Node *next = NULL );

    // Set and get the next node
    Node *next();
    Node *prev();
    void set_next( Node *next );
    void set_prev( Node *prev );
};


class List{
  protected:
    unsigned _count;
    Node *_first;
    Node *_current;
    Node *_last;

  public:
    // Constructor
    List();

    // Getting nodes
    Node *current();
    Node *get_first();
    Node *get_last();

    // Setting nodes
    Node *first();
    Node *last();
    Node *next();
    Node *prev();
    unsigned count();

    // Adding and deleting nodes
    void append( Node &n );   // Appends at the end of the list
    void insert( Node &n );   // Does an insert after
    void insert_before( Node &n );
    void insert_after( Node &n );
    Node *remove();
};

/***************************************************************************/


class INode:public Node{
  protected:
    int _value;

  public:
    // Constructor
    INode( int val = 0, INode *prev = NULL, INode *next = NULL );

    // retrieve the value
    int value();
};

#endif

/*
 * $Log: list.H,v $
 * Revision 1.2  1994/08/24  14:35:27  asf
 * Added insert_before() and insert_after()
 *
 * Revision 1.1  1994/07/26  20:06:38  asf
 * Initial revision
 *
 */
